//===----- ISLQueryBuilder.h - Querying Interface for ISLValueAnalysis ----===//
//
//===----------------------------------------------------------------------===//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// Copyright (c) 2015 Advanced Micro Devices, Inc.  All rights reserved.
// Developed by: Advanced Micro Devices, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// with the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// Redistributions of source code must retain the above copyright notice, this
// list of conditions and the following disclaimers.
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimers in the documentation
// and/or other materials provided with the distribution.
// Neither the names of Advanced Micro Devices, Inc., nor the names of its
// contributors may be used to endorse or promote products derived from this
// Software without specific prior written permission.
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH
// THE SOFTWARE.
//
//===----------------------------------------------------------------------===//
//
//===----------------------------------------------------------------------===//

#ifndef LLVM_TRANSFORMS_IPO_ISLQUERYBUILDER_H
#define LLVM_TRANSFORMS_IPO_ISLQUERYBUILDER_H

#include "llvm/Analysis/ISLValueAnalysis.h"

namespace llvm {

class QueryNode {
public:
  using IslMapVec = ISLFunctionInfo::IslMapVec;

  Value *V;
  IslMapVec MapVec;

  QueryNode(Value *V, IslMapVec &&MapVec) : V(V), MapVec(MapVec) {}

  ~QueryNode() {
    std::for_each(MapVec.begin(), MapVec.end(),
                  [](isl_map *&M) { isl_map_free(M); });
  }

  isl_map* getMapAt(unsigned Idx) {
    return isl_map_copy(MapVec[Idx]);
  }

};

class ISLQueryBuilder {
public:
  using IslMapVec = ISLFunctionInfo::IslMapVec;
  using OpVec = ISLFunctionInfo::OpVec;
  using OpVecVec = ISLFunctionInfo::OpVecVec;

  Function *F;
  ISLFunctionInfo *FI;

  enum Tribool {
    True = 1,
    False = 0,
    Unknown = -1
  };

  // Constructor
  ISLQueryBuilder(Function *F, ISLFunctionInfo *FI) : F(F), FI(FI) {}

  // Create Dim based OperandInfo
  OperandInfo createDimOperandInfo(unsigned DimNum);
  // Create Const based OperandInfo
  OperandInfo createConstOperandInfo(unsigned Const);

  // Create <Dim-Dim> vector operands for size
  OpVecVec createTwoDimVectorOperands(unsigned VectorSize);
  // Create <Dim-Const> vector operands for size
  OpVecVec createDimConstVectorOperands(unsigned VectorSize, long Const);
  // Create <Const-Dim> vector operands for size
  OpVecVec createConstDimVectorOperands(unsigned VectorSize, long Const);
  // Create QueryNode
  QueryNode getQueryNode(Value *V);
  // Perform QN * Const
  QueryNode mulQueryNodeWithConst(QueryNode &QN, long Const);
  // Perform QN / Const
  QueryNode divQueryNodeWithConst(QueryNode &QN, long Const);
  // Perform QN1 OpCode QN2
  QueryNode binOpQueryNodes(QueryNode &QN1, QueryNode &QN2, unsigned OpCode,
                            unsigned NBits);
  // Perform QN1 CmpPredicate QN2
  Tribool isQueryNodesCmpEQ(CmpInst::Predicate Predicate, QueryNode &QN1,
                         QueryNode &QN2);
};
} // end of llvm namespace

#endif // LLVM_TRANSFORMS_IPO_ISLQUERYBUILDER_H
