//===- LegacyAliasAnalysis.h - AA Wrapper for Legacy Passes ------------===//
//* Some Legal Stuff
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//====  Copyright (c) 2015 Advanced Micro Devices, Inc.  All rights reserved.
//
//               Developed by: Advanced Micro Devices, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// with the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// Redistributions of source code must retain the above copyright notice, this
// list of conditions and the following disclaimers.
//
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimers in the documentation
// and/or other materials provided with the distribution.
//
// Neither the names of Advanced Micro Devices, Inc., nor the names of its
// contributors may be used to endorse or promote products derived from this
// Software without specific prior written permission.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH
// THE SOFTWARE.
//===----------------------------------------------------------------------===//
#ifndef LLVM_LEGACY_ALIAS_ANALYSIS_H
#define LLVM_LEGACY_ALIAS_ANALYSIS_H

#include "llvm/Analysis/BasicAliasAnalysis.h"
#include "llvm/Analysis/TypeBasedAliasAnalysis.h"
#include "llvm/Analysis/ScalarEvolutionAliasAnalysis.h"

namespace llvm
{
/// This class is a functor to be used in legacy module or SCC passes for
/// computing AA results for a function. We store the results in fields so that
/// they live long enough to be queried, but we re-use them each time.
class LegacyAARGetterX {
  Pass &P;
  Optional<BasicAAResult> BAR;
  Optional<TypeBasedAAResult> TAR;
  Optional<SCEVAAResult> SAR;
  Optional<AAResults> AAR;
public:
  LegacyAARGetterX(Pass &P) : P(P) {}
  AAResults &operator()(Function &F);
  static void getAnalysisUsage(AnalysisUsage& AU);
};

#define INITIALIZE_PASS_DEPENDENCY_LEGACY_AA \
  if (FreshAAResults) { \
    INITIALIZE_PASS_DEPENDENCY(LoopInfoWrapperPass) \
    INITIALIZE_PASS_DEPENDENCY(AssumptionCacheTracker) \
    INITIALIZE_PASS_DEPENDENCY(TargetLibraryInfoWrapperPass) \
    INITIALIZE_PASS_DEPENDENCY(ScalarEvolutionWrapperPass) \
    INITIALIZE_PASS_DEPENDENCY(BasicAAWrapperPass) \
    INITIALIZE_PASS_DEPENDENCY(SCEVAAWrapperPass) \
    INITIALIZE_PASS_DEPENDENCY(TypeBasedAAWrapperPass) \
  }
  extern cl::opt<bool> FreshAAResults;
}
#endif
