//===-- VirtualRegisterPressureAnalyzer.h -
//                  Estimate register pressure for LICM--*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===--------------------------------------------------------------===//
//
// This file declares routines for estimating the register pressure for
// performing loop invariant code motion.
// This utility is based on the paper
// Controlling Virtual Register Pressure in LLVM Middle-End by Ivan Baev
//
//===--------------------------------------------------------------===//
//===----------------------------------------------------------------------===//
//====  Copyright (c) 2015 Advanced Micro Devices, Inc.  All rights reserved.
//
//               Developed by: Advanced Micro Devices, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// with the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// Redistributions of source code must retain the above copyright notice, this
// list of conditions and the following disclaimers.
//
// Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimers in the documentation
// and/or other materials provided with the distribution.
//
// Neither the names of Advanced Micro Devices, Inc., nor the names of its
// contributors may be used to endorse or promote products derived from this
// Software without specific prior written permission.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH
// THE SOFTWARE.

#ifndef LLVM_ANALYSIS_VIRTUALREGISTERPRESSUREANALYZER_H
#define LLVM_ANALYSIS_VIRTUALREGISTERPRESSUREANALYZER_H

namespace llvm {
class AAResults;
class AliasSetTracker;
class DominatorTree;
struct LoopSafetyInfo;
class OptimizationRemarkEmitter;
class LoopUtils;
class Loop;
class TargetTransformInfo;
class Instruction;
class TargetLibraryInfo;
class Type;
class Value;
class DataLayout;

bool IfHoistableInstruction(Instruction &I);
void estimateRegisterPressure(Loop *CurLoop, TargetTransformInfo *TTI,
                              int &PermissibleIntLIs, int &PermissibleFloatLIs);
bool CheckWholeLoopInvariance(Loop *CurLoop);

}

#endif
